// ===========================================================================
// CONTENT  : INTERFACE OpenIdConnectConstants
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.1 - 29/12/2016
// HISTORY  :
//  18/08/2014  mdu  CREATED
//  29/12/2016  mdu   added --> OP configuration discovery constants
//
// Copyright (c) 2014-2016, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.oidc ;

import org.pfsw.oauth.OAuth2Constants;

/**
 * Various constants for elements defined in the OpenID Connect Core 1.0
 * specification. 
 *
 * @author Manfred Duchrow
 * @version 1.1
 */
public interface OpenIdConnectConstants
{ 
  // ===========================================================================
  // OpenID Connect Core 1.0 - 2. ID Token
  // ===========================================================================
  public static final String ID_TOKEN_ISSUER = "iss";
  public static final String ID_TOKEN_SUBJECT = "sub";
  public static final String ID_TOKEN_AUDIENCE = "aud";
  public static final String ID_TOKEN_EXPIRES = "exp";
  public static final String ID_TOKEN_ISSUED_AT = "iat";
  public static final String ID_TOKEN_AUTH_TIME = "auth_time";
  public static final String ID_TOKEN_NONCE = "nonce";
  public static final String ID_TOKEN_AUTH_CONTEXT_REF = "acr";
  public static final String ID_TOKEN_AUTH_METHOD_REF = "amr";
  public static final String ID_TOKEN_AUTHORIZED_PARTY = "azp";
  
  // ===========================================================================
  // OpenID Connect Core 1.0 - 3.1.2.1.  Authentication Request
  // ===========================================================================
  /**
   * scope
   * REQUIRED. 
   * OpenID Connect requests MUST contain the openid scope value. If the openid 
   * scope value is not present, the behavior is entirely unspecified. Other scope 
   * values MAY be present. Scope values used that are not understood by an 
   * implementation SHOULD be ignored. See Sections 5.4 and 11 for additional scope 
   * values defined by this specification. 
   */
  public static final String AUTH_REQ_PARAM_SCOPE = OAuth2Constants.SCOPE;

  /**
   * response_type
   * REQUIRED. 
   * OAuth 2.0 Response Type value that determines the authorization processing flow 
   * to be used, including what parameters are returned from the endpoints used. 
   * When using the Authorization Code Flow, this value is "code". 
   */
  public static final String AUTH_REQ_PARAM_RESPONSE_TYPE = OAuth2Constants.RESPONSE_TYPE;

  /**
   * client_id
   * REQUIRED. 
   * OAuth 2.0 Client Identifier valid at the Authorization Server. 
   */
  public static final String AUTH_REQ_PARAM_CLIENT_ID = OAuth2Constants.CLIENT_ID;

  /**
   * redirect_uri
   * REQUIRED. 
   * Redirection URI to which the response will be sent. This URI MUST exactly match 
   * one of the Redirection URI values for the Client pre-registered at the OpenID 
   * Provider, with the matching performed as described in Section 6.2.1 of [RFC3986] 
   * (Simple String Comparison).
   */
  public static final String AUTH_REQ_PARAM_REDIRECT_URI = OAuth2Constants.REDIRECT_URI;

  /**
   * state
   * RECOMMENDED. 
   * Opaque value used to maintain state between the request and the callback. Typically, 
   * Cross-Site Request Forgery (CSRF, XSRF) mitigation is done by cryptographically binding 
   * the value of this parameter with a browser cookie. 
   */
  public static final String AUTH_REQ_PARAM_STATE = OAuth2Constants.STATE;

  /**
   * nonce
   * OPTIONAL. 
   * String value used to associate a Client session with an ID Token, and to mitigate 
   * replay attacks. The value is passed through unmodified from the Authorization 
   * Request to the ID Token.
   */
  public static final String AUTH_REQ_PARAM_NONCE = "nonce";
  
  /**
   * display
   * OPTIONAL. 
   * ASCII string value that specifies how the Authorization Server displays the 
   * authentication and consent user interface pages to the End-User.
   */
  public static final String AUTH_REQ_PARAM_DISPLAY = "display";
  
  /**
   * prompt
   * OPTIONAL. 
   * Space delimited, case sensitive list of ASCII string values that specifies 
   * whether the Authorization Server prompts the End-User for re-authentication and 
   * consent.
   */
  public static final String AUTH_REQ_PARAM_PROMPT = "prompt";

  // Defined parameter values
  
  public static final String SCOPE_OPENID = "openid";

  public static final String RESPONSE_TYPE_CODE = "code";

  public static final String AUTH_PROOF_PARAM = "code";

  public static final String AUTH_STATE_PARAM = "state";

  // ===========================================================================
  // OpenID Connect Core 1.0 - 3.1.3.1.  Token Request
  // ===========================================================================
  public static final String TOKEN_REQ_PARAM_GRANT_TYPE = OAuth2Constants.GRANT_TYPE;

  public static final String TOKEN_REQ_PARAM_CODE = OAuth2Constants.CODE;

  public static final String TOKEN_REQ_PARAM_CLAIMS = "claims";

  public static final String TOKEN_REQ_PARAM_REDIRECT_URI = "redirect_uri";
 
  public static final String TOKEN_REQ_PARAM_REFRESH_TOKEN = OAuth2Constants.REFRESH_TOKEN;

  public static final String TOKEN_REQ_PARAM_SCOPE = OAuth2Constants.SCOPE;

  public static final String TOKEN_REQ_PARAM_PROMPT = "prompt";

  public static final String TOKEN_REQ_PARAM_CLIENT_ID = OAuth2Constants.CLIENT_ID;

  public static final String TOKEN_REQ_PARAM_CLIENT_SECRET = OAuth2Constants.CLIENT_SECRET;

  // Defined parameter values
  public static final String GRANT_TYPE_AUTHORIZATION_CODE = "authorization_code";
  public static final String GRANT_TYPE_REFRESH_TOKEN = "refresh_token";

  public static final String SCOPE_OPENID_PROFILE = "openid profile";

  // ===========================================================================
  // OpenID Connect Core 1.0 - 3.1.3.3.  Successful Token Response
  // ===========================================================================
  public static final String TOKEN_RESP_ACCESS_TOKEN = OAuth2Constants.ACCESS_TOKEN;

  public static final String TOKEN_RESP_TOKEN_TYPE = OAuth2Constants.TOKEN_TYPE;

  public static final String TOKEN_RESP_REFRESH_TOKEN = OAuth2Constants.REFRESH_TOKEN;

  public static final String TOKEN_RESP_EXPIRES_IN = "expires_in"; // seconds
  
  public static final String TOKEN_RESP_ID_TOKEN = "id_token";

  public static final String TOKEN_RESP_SCOPE = "scope";
  
  public static final String TOKEN_RESP_STATE = "state";
  
  // Defined parameter values
  public static final String TOKEN_RESP_TOKEN_TYPE_VALUE_1 = "Bearer";
  
  // ===========================================================================
  // OpenID Connect Core 1.0 - 3.1.3.4.  Token Error Response
  // ===========================================================================
  public static final String TOKEN_RESP_ERROR = "error";

  // Defined parameter values
  public static final String TOKEN_RESP_ERROR_VALUE_1 = "invalid_request";

  // ===========================================================================
  // OpenID Connect Core 1.0 - 5.1.  Standard Claims
  // ===========================================================================
  /**
   * Subject - Identifier for the End-User at the Issuer.
   */
  public static final String CLAIM_SUBJECT = "sub";
  /**
   * End-User's full name in displayable form including all name parts, possibly 
   * including titles and suffixes, ordered according to the End-User's locale and 
   * preferences. 
   */
  public static final String CLAIM_NAME = "name";
  /**
   * Given name(s) or first name(s) of the End-User. Note that in some cultures, 
   * people can have multiple given names; all can be present, with the names being 
   * separated by space characters. 
   */
  public static final String CLAIM_GIVEN_NAME = "given_name";
  /**
   * Surname(s) or last name(s) of the End-User. Note that in some cultures, 
   * people can have multiple family names or no family name; all can be present, 
   * with the names being separated by space characters.
   */
  public static final String CLAIM_FAMILY_NAME = "family_name";
  /**
   * Middle name(s) of the End-User. Note that in some cultures, people can have 
   * multiple middle names; all can be present, with the names being separated by 
   * space characters. Also note that in some cultures, middle names are not used. 
   */
  public static final String CLAIM_MIDDLE_NAME = "middle_name";
  /**
   * Casual name of the End-User that may or may not be the same as the given_name. 
   * For instance, a nickname value of Mike might be returned alongside a given_name 
   * value of Michael.
   */
  public static final String CLAIM_NICKNAME = "nickname";
  /**
   * Shorthand name by which the End-User wishes to be referred to at the RP, such as 
   * janedoe or j.doe. This value MAY be any valid JSON string including special characters 
   * such as @, /, or whitespace. The RP MUST NOT rely upon this value being unique.
   */
  public static final String CLAIM_PREFERRED_NAME = "preferred_username";
  /**
   * URL of the End-User's profile page. The contents of this Web page SHOULD be about the End-User. 
   */
  public static final String CLAIM_PROFILE_URL = "profile";
  /**
   * URL of the End-User's profile picture. This URL MUST refer to an image file (for example, 
   * a PNG, JPEG, or GIF image file), rather than to a Web page containing an image. Note that 
   * this URL SHOULD specifically reference a profile photo of the End-User suitable for displaying 
   * when describing the End-User, rather than an arbitrary photo taken by the End-User. 
   */
  public static final String CLAIM_PICTURE_URL = "picture";
  /**
   * URL of the End-User's Web page or blog. This Web page SHOULD contain information published by 
   * the End-User or an organization that the End-User is affiliated with. 
   */
  public static final String CLAIM_WEBSITE_URL = "website";
  /**
   * End-User's preferred e-mail address. Its value MUST conform to the 
   * RFC 5322 [RFC5322] addr-spec syntax. 
   * The RP MUST NOT rely upon this value being unique.
   */
  public static final String CLAIM_EMAIL = "email";
  /**
   * True if the End-User's e-mail address has been verified; otherwise false. 
   * When this Claim Value is true, this means that the OP took affirmative steps 
   * to ensure that this e-mail address was controlled by the End-User at the time 
   * the verification was performed. The means by which an e-mail address is verified 
   * is context-specific, and dependent upon the trust framework or contractual 
   * agreements within which the parties are operating. 
   */
  public static final String CLAIM_EMAIL_VERIFIED = "email_verified";
  /**
   * End-User's gender. Values defined by this specification are female and male. 
   * Other values MAY be used when neither of the defined values are applicable.
   */
  public static final String CLAIM_GENDER = "gender";
  /**
   * End-User's birthday, represented as an ISO 8601:2004 [ISO8601‑2004] YYYY-MM-DD format. 
   * The year MAY be 0000, indicating that it is omitted. To represent only the year, 
   * YYYY format is allowed. Note that depending on the underlying platform's date related 
   * function, providing just year can result in varying month and day, so the implementers 
   * need to take this factor into account to correctly process the dates. 
   */
  public static final String CLAIM_BIRTHDATE = "birthdate";
  /**
   * String from zoneinfo [http://www.twinsun.com/tz/tz-link.htm] time zone database representing 
   * the End-User's time zone. 
   * For example, Europe/Paris or America/Los_Angeles.
   */
  public static final String CLAIM_ZONE_INFO = "zoneinfo";
  /**
   * End-User's locale, represented as a BCP47 [RFC5646] language tag. 
   * This is typically an ISO 639-1 Alpha-2 [ISO639‑1] language code in lowercase and an 
   * ISO 3166-1 Alpha-2 [ISO3166‑1] country code in uppercase, separated by a dash. 
   * For example, en-US or fr-CA. As a compatibility note, some implementations have used 
   * an underscore as the separator rather than a dash, for example, en_US; Relying Parties 
   * MAY choose to accept this locale syntax as well.
   */
  public static final String CLAIM_LOCALE = "locale";
  /**
   * End-User's preferred telephone number. 
   * E.164 [http://www.itu.int/rec/T-REC-E.164-201011-I/en] is RECOMMENDED as the format of 
   * this Claim, for example, +1 (425) 555-1212 or +56 (2) 687 2400. 
   * If the phone number contains an extension, it is RECOMMENDED that the extension be 
   * represented using the RFC 3966 [RFC3966] extension syntax, for example, +1 (604) 555-1234;ext=5678. 
   */
  public static final String CLAIM_PHONE_NUMBER = "phone_number";
  /**
   * True if the End-User's phone number has been verified; otherwise false. 
   * When this Claim Value is true, this means that the OP took affirmative steps to ensure 
   * that this phone number was controlled by the End-User at the time the verification 
   * was performed. 
   * The means by which a phone number is verified is context-specific, and dependent upon 
   * the trust framework or contractual agreements within which the parties are operating. 
   * When true, the phone_number Claim MUST be in E.164 format and any extensions MUST be 
   * represented in RFC 3966 format. 
   */
  public static final String CLAIM_PHONE_NUMBER_VERIFIED = "phone_number_verified";
  /**
   * End-User's preferred postal address. The value of the address member is a JSON [RFC4627] 
   * structure containing some or all of the members defined in Section 5.1.1.
   */
  public static final String CLAIM_ADDRESS = "address";
  /**
   * Time the End-User's information was last updated. Its value is a JSON number representing 
   * the number of seconds from 1970-01-01T0:0:0Z as measured in UTC until the date/time. 
   */
  public static final String CLAIM_UPDATED_AT = "updated_at";
  
  // ===========================================================================
  // OpenID Connect Core 1.0 - 5.1.1.  Address Claim
  // ===========================================================================
  /**
   * Full mailing address, formatted for display or use on a mailing label. 
   * This field MAY contain multiple lines, separated by newlines. 
   * Newlines can be represented either as a carriage return/line feed pair ("\r\n") 
   * or as a single line feed character ("\n"). 
   */
  public static final String CLAIM_ADDRESS_FORMATTED = "formatted";
  /**
   * Full street address component, which MAY include house number, street name, 
   * Post Office Box, and multi-line extended street address information. 
   * This field MAY contain multiple lines, separated by newlines. 
   * Newlines can be represented either as a carriage return/line feed pair ("\r\n") 
   * or as a single line feed character ("\n"). 
   */
  public static final String CLAIM_ADDRESS_STREET = "street_address";
  /**
   * City or locality component. 
   */
  public static final String CLAIM_ADDRESS_LOCALITY = "locality";
  /**
   * State, province, prefecture, or region component. 
   */
  public static final String CLAIM_ADDRESS_REGION = "region";
  /**
   * Zip code or postal code component. 
   */
  public static final String CLAIM_ADDRESS_POSTAL_CODE = "postal_code";
  /**
   * Country name component. 
   */
  public static final String CLAIM_ADDRESS_COUNTRY = "country";
  // ===========================================================================
  // OpenID Connect Session Management 1.0
  // http://openid.net/specs/openid-connect-session-1_0.html
  // ===========================================================================
  public static final String SESSION_STATE = "session_state";
  
  // ===========================================================================
  // OpenID Connect Back-Channel Logout 1.0 LogoutToken
  // http://openid.net/specs/openid-connect-backchannel-1_0.html#LogoutToken
  // ===========================================================================
  /**
   * Back-channel logout token field "iss". 
   */
  public static final String BCLO_TOKEN_ISSUER = "iss";
  /**
   * Back-channel logout token field "sub". 
   */
  public static final String BCLO_TOKEN_SUBJECT = "sub";
  /**
   * Back-channel logout token field "aud". 
   */
  public static final String BCLO_TOKEN_AUDIENCE = "aud";
  /**
   * Back-channel logout token field "jti". 
   */
  public static final String BCLO_TOKEN_JTI = "jti";
  /**
   * Back-channel logout token field "events". 
   */
  public static final String BCLO_TOKEN_EVENTS = "events";
  /**
   * Back-channel logout token field "sid". 
   */
  public static final String BCLO_TOKEN_SESSION_ID = "sid";

  /**
   * OpenID Event URN for a back-channel logout. 
   */
  public static final String EVENT_BACKCHANNEL_LOGOUT = "http://schemas.openid.net/event/backchannel-logout";

  // ===========================================================================
  // ENDPOINT SUFFIXES
  // ===========================================================================
  /**
   * The endpoint suffix of the OpenID Connect configuration discovery request.
   * See OpenID Connect Discovery 1.0 (4. Obtaining OpenID Provider Configuration Information).
   * http://openid.net/specs/openid-connect-discovery-1_0.html 
   */
  public static final String URI_SUFFIX_CONFIG_DISCOVERY = "/.well-known/openid-configuration";
  /**
   * The (non normative) default URL endpoint suffix of the OpenID Connect authentication request.
   * See OpenID Connect Core 1.0 (3.1.2.1. Authentication Request). 
   */
  public static final String DEFAULT_URI_SUFFIX_AUTHENTICATION_REQUEST = "/authorize";
  /**
   * The (non normative) default URL endpoint suffix of the OpenID Connect token request.
   * See OpenID Connect Core 1.0 (3.1.3.1. Token Request). 
   */
  public static final String DEFAULT_URI_SUFFIX_TOKEN_REQUEST = "/token";
  /**
   * The (non normative) default URL endpoint suffix of the OpenID Connect user info request.
   * See OpenID Connect Core 1.0 (5.3.1. UserInfo Request). 
   */
  public static final String DEFAULT_URI_SUFFIX_USERINFO_REQUEST = "/userinfo";

  // ===========================================================================
  // CONFIGURATION DISCOVERY 
  // (3.OpenID Provider Metadata http://openid.net/specs/openid-connect-discovery-1_0.html#ProviderMetadata)
  // ===========================================================================
  /**
   * The issuer field of the OIDC configuration discovery response.
   */
  public static final String OP_META_ISSUER = "issuer";
  /**
   * The authorization_endpoint field of the OIDC configuration discovery response.
   */
  public static final String OP_META_AUTHORIZATION_ENDPOINT = "authorization_endpoint";
  /**
   * The token_endpoint field of the OIDC configuration discovery response.
   */
  public static final String OP_META_TOKEN_ENDPOINT = "token_endpoint";
  /**
   * The userinfo_endpoint field of the OIDC configuration discovery response.
   */
  public static final String OP_META_USERINFO_ENDPOINT = "userinfo_endpoint";
  /**
   * The registration_endpoint field of the OIDC configuration discovery response.
   */
  public static final String OP_META_REGISTRATION_ENDPOINT = "registration_endpoint";
  /**
   * The jwks_uri field of the OIDC configuration discovery response.
   */
  public static final String OP_META_JWKS_URI = "jwks_uri";
  /**
   * The scopes_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_SCOPES_SUPPORTED = "scopes_supported";
  /**
   * The response_types_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_RESPONSE_TYPES_SUPPORTED = "response_types_supported";
  /**
   * The response_modes_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_RESPONSE_MODES_SUPPORTED = "response_modes_supported";
  /**
   * The grant_types_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_GRANT_TYPES_SUPPORTED = "grant_types_supported";
  /**
   * The acr_values_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_ACR_VALUES_SUPPORTED = "acr_values_supported";
  /**
   * The subject_types_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_SUBJECT_TYPES_SUPPORTED = "subject_types_supported";
  /**
   * The id_token_signing_alg_values_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_ID_TOKEN_SIGNING_ALG_VALUES_SUPPORTED = "id_token_signing_alg_values_supported";
  /**
   * The id_token_encryption_alg_values_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_ID_TOKEN_ENCRYPTION_ALG_VALUES_SUPPORTED = "id_token_encryption_alg_values_supported";
  /**
   * The id_token_encryption_enc_values_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_ID_TOKEN_ENCRYPTION_ENC_VALUES_SUPPORTED = "id_token_encryption_enc_values_supported";
  /**
   * The claim_types_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_CLAIM_TYPES_SUPPORTED = "claim_types_supported";
  /**
   * The claims_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_CLAIMS_SUPPORTED = "claims_supported";
  /**
   * The claims_parameter_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_CLAIMS_PARAMETER_SUPPORTED = "claims_parameter_supported";
  /**
   * The request_parameter_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_REQUEST_PARAMETER_SUPPORTED = "request_parameter_supported";

  // ===== From http://openid.net/specs/openid-connect-session-1_0.html#OPMetadata =====
  /**
   * The check_session_iframe field of the OIDC configuration discovery response.
   */
  public static final String OP_META_CHECK_SESSION_IFRAME = "check_session_iframe";
  /**
   * The end_session_endpoint field of the OIDC configuration discovery response.
   */
  public static final String OP_META_END_SESSION_ENDPOINT = "end_session_endpoint";

  // ===== From http://openid.net/specs/openid-connect-frontchannel-1_0.html =====
  /**
   * The frontchannel_logout_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_FRONTCHANNEL_LOGOUT_SUPPORTED = "frontchannel_logout_supported";
  /**
   * The frontchannel_logout_session_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_FRONTCHANNEL_LOGOUT_SESSION_SUPPORTED = "frontchannel_logout_session_supported";

  // ===== From http://openid.net/specs/openid-connect-backchannel-1_0.html =====
  /**
   * The backchannel_logout_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_BACKCHANNEL_LOGOUT_SUPPORTED = "backchannel_logout_supported";
  /**
   * The backchannel_logout_session_supported field of the OIDC configuration discovery response.
   */
  public static final String OP_META_BACKCHANNEL_LOGOUT_SESSION_SUPPORTED = "backchannel_logout_session_supported";
}