// ===========================================================================
// CONTENT  : CLASS OpenIdProviderMetadata
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 29/12/2016
// HISTORY  :
//  29/12/2016  mdu  CREATED
//
// Copyright (c) 2016, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.oidc;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.pfsw.oidc.OpenIdConnectConstants.*;

import java.util.List;

import org.pfsw.text.json.JsonArray;
import org.pfsw.text.json.JsonObject;

/**
 * An instance of this class contains the response data of a metadata discovery
 * request as specified in http://openid.net/specs/openid-connect-discovery-1_0.html
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class OpenIdProviderMetadata
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private final JsonObject metadata;

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  public OpenIdProviderMetadata()
  {
    this(new JsonObject());
  }
  
  public OpenIdProviderMetadata(JsonObject metadata)
  {
    super();
    if (metadata == null)
    {
      throw new IllegalArgumentException("The provided metadata JSON object must not be null!");
    }
    this.metadata = metadata;
  }

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  public JsonObject getMetadata()
  {
    return this.metadata;
  }
  
  public String getIssuer()
  {
    return this.getStringField(OP_META_ISSUER);
  }
  
  public String getAuthorizationEndpoint()
  {
    return this.getStringField(OP_META_AUTHORIZATION_ENDPOINT);
  }

  public String getTokenEndpoint()
  {
    return this.getStringField(OP_META_TOKEN_ENDPOINT);
  }

  public String getUserInfoEndpoint()
  {
    return this.getStringField(OP_META_USERINFO_ENDPOINT);
  }
  
  public String getRegistrationEndpoint()
  {
    return this.getStringField(OP_META_REGISTRATION_ENDPOINT);
  }
  
  public String getJsonWebKeySetURI()
  {
    return this.getStringField(OP_META_JWKS_URI);
  }
  
  public String getEndSessionEndpoint()
  {
    return this.getStringField(OP_META_END_SESSION_ENDPOINT);
  }
  
  public List<String> getGrantTypesSupported()
  {
    return this.getStringsField(OP_META_GRANT_TYPES_SUPPORTED);
  }
  
  public List<String> getResponseTypesSupported()
  {
    return this.getStringsField(OP_META_RESPONSE_TYPES_SUPPORTED);
  }
  
  public List<String> getScopesSupported()
  {
    return this.getStringsField(OP_META_SCOPES_SUPPORTED);
  }
  
  public List<String> getSubjectTypesSupported()
  {
    return this.getStringsField(OP_META_SUBJECT_TYPES_SUPPORTED);
  }
  
  public List<String> getIdTokenSigningAlgorithmsSupported()
  {
    return this.getStringsField(OP_META_ID_TOKEN_SIGNING_ALG_VALUES_SUPPORTED);
  }
  
  public List<String> getClaimsSupported()
  {
    return this.getStringsField(OP_META_CLAIMS_SUPPORTED);
  }
  
  public Boolean getClaimsParameterSupported()
  {
    return this.getBooleanField(OP_META_CLAIMS_PARAMETER_SUPPORTED);
  }
  
  public String getStringField(String name)
  {
    return this.getMetadata().getString(name);
  }

  public List<String> getStringsField(String name)
  {
    JsonArray jsonArray;
    
    jsonArray = this.getMetadata().getJsonArray(name);
    if (jsonArray == null)
    {
      return null;
    }
    return jsonArray.asList(String.class);
  }

  public Boolean getBooleanField(String name)
  {
    return this.getMetadata().getBoolean(name);
  }
}

