// ===========================================================================
// CONTENT  : CLASS IdToken
// AUTHOR   : Manfred Duchrow
// VERSION  : 2.0 - 16/02/2017
// HISTORY  :
//  22/08/2014  mdu  CREATED
//  31/12/2016  mdu   added -> implements Serializable
//  16/02/2017  mdu   changed -> renamed, added header
//
// Copyright (c) 2014-2017, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.oidc.token;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.io.Serializable;

import org.pfsw.jwt.jws.JoseHeader;
import org.pfsw.text.StringUtil;
import org.pfsw.text.TimeValue;
import org.pfsw.text.json.JsonObject;

/**
 * Represents a JSON Web Token (JWT) according to section "2. ID Token"
 * of OpenID Connect Core 1.0 specification.
 *
 * @author Manfred Duchrow
 * @version 2.0
 */
public class IdToken implements Serializable
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  private static final long serialVersionUID = -6335454121041406466L;
  private static final StringUtil SU = StringUtil.current();

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private JoseHeader header;
  private String issuer;
  private String subject;
  private String[] audiencesArray = StringUtil.EMPTY_STRING_ARRAY;
  private TimeValue expirationTime; // milliseconds since 1970-01-01T0:0:0Z
  private TimeValue issuedTime; // milliseconds since 1970-01-01T0:0:0Z
  private TimeValue authenticaionTime; // milliseconds since 1970-01-01T0:0:0Z
  private String nonce;
  private String authenticationContext;
  private String[] authenticationMethodsArray = StringUtil.EMPTY_STRING_ARRAY;
  private String authorizedParty;

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  public IdToken()
  {
    super();
  }

  public IdToken(JsonObject header)
  {
    super();
    this.setHeader(new JoseHeader(header));
  }
  
  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  public JoseHeader getHeader()
  {
    return this.header;
  }
  
  public void setHeader(JoseHeader header)
  {
    this.header = header;
  }
  
  public String getIssuer()
  {
    return issuer;
  }

  public void setIssuer(String issuer)
  {
    this.issuer = issuer;
  }

  public String getSubject()
  {
    return subject;
  }

  public void setSubject(String subject)
  {
    this.subject = subject;
  }

  /**
   * Returns the time since 01.01.1970.
   */
  public TimeValue getExpirationTime()
  {
    return this.expirationTime;
  }

  /**
   * Sets the time since 01.01.1970.
   */
  public void setExpirationTime(TimeValue expirationTime)
  {
    this.expirationTime = expirationTime;
  }

  /**
   * Returns the time since 01.01.1970.
   */
  public TimeValue getIssuedTime()
  {
    return this.issuedTime;
  }

  /**
   * Sets the time since 01.01.1970.
   */
  public void setIssuedTime(TimeValue issuedTime)
  {
    this.issuedTime = issuedTime;
  }

  /**
   * Returns the time since 01.01.1970.
   */
  public TimeValue getAuthenticaionTime()
  {
    return this.authenticaionTime;
  }

  /**
   * Sets the time in milliseconds since 01.01.1970.
   */
  public void setAuthenticationTime(TimeValue authenticaionTime)
  {
    this.authenticaionTime = authenticaionTime;
  }

  public String getNonce()
  {
    return nonce;
  }

  public void setNonce(String nonce)
  {
    this.nonce = nonce;
  }

  public String getAuthenticationContext()
  {
    return authenticationContext;
  }

  public void setAuthenticationContext(String authenticationContext)
  {
    this.authenticationContext = authenticationContext;
  }

  public String getAuthorizedParty()
  {
    return authorizedParty;
  }

  public void setAuthorizedParty(String authorizedParty)
  {
    this.authorizedParty = authorizedParty;
  }
  
  public boolean isExpired() 
  {
    return System.currentTimeMillis() >= this.getExpirationTime().asMilliseconds();
  }

  /**
   * Returns all defined audiences or an empty array (never null).
   */
  public String[] getAudiences()
  {
    return SU.copy(this.getAudiencesArray());
  }
  
  /**
   * Returns all defined authentication methods or an empty array (never null).
   */
  public String[] getAuthenticationMethods()
  {
    return SU.copy(this.getAuthenticationMethodsArray());
  }
  
  /**
   * Add one or more Audiences (aud).
   */
  public void addAudiences(String... audienceIds)
  {
    this.setAudiencesArray(SU.appendIfNotThere(this.getAudiencesArray(), audienceIds));
  }
  
  /**
   * Add one or more Authentication Method References (amr).
   */
  public void addAuthenticationMethods(String... methods)
  {
    this.setAuthenticationMethodsArray(SU.appendIfNotThere(this.getAuthenticationMethodsArray(), methods));
  }
  
  @Override
  public String toString()
  {
    return "IdToken(" + "issuer: " + this.getIssuer() + " ; subject: " + this.getSubject() + ")";
  }

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected String[] getAudiencesArray()
  {
    return audiencesArray;
  }
  
  protected void setAudiencesArray(String[] array)
  {
    this.audiencesArray = array;
  }
  
  protected String[] getAuthenticationMethodsArray()
  {
    return authenticationMethodsArray;
  }
  
  public void setAuthenticationMethodsArray(String[] array)
  {
    this.authenticationMethodsArray = array;
  }

}
