// ===========================================================================
// CONTENT  : CLASS IdTokenConverter
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.0 - 16/02/2017
// HISTORY  :
//  16/02/2017  mdu  CREATED
//
// Copyright (c) 2017, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.oidc.token;

// ===========================================================================
// IMPORTS
// ===========================================================================
import org.pfsw.jwt.BaseJsonWebTokenConverter;
import org.pfsw.jwt.JwtConversionException;
import org.pfsw.oidc.OpenIdConnectConstants;
import org.pfsw.text.StringUtil;
import org.pfsw.text.json.JsonArray;
import org.pfsw.text.json.JsonObject;
import org.pfsw.text.json.JsonObjectAccessor;

/**
 * Converter for OIDC specific objects.
 *
 * @author Manfred Duchrow
 * @version 1.0
 */
public class IdTokenConverter extends BaseJsonWebTokenConverter<IdToken>
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  private static final StringUtil SU = StringUtil.current();

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  public IdTokenConverter()
  {
    super();
  }

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  @Override
  protected IdToken createToken(JsonObject headerObject, JsonObject payloadObject, byte[] signature) throws JwtConversionException
  {
    return this.convertToIdTokenClaims(headerObject, payloadObject);
  }
  
  /**
   * Converts the given JsonObject to a JSON Web Token (JWT) according to the
   * OpenID Connect Core 1.0 specification, section 2.  ID Token.
   * 
   * @param payloadObject An object that must at least contain the required ID Token claims. 
   * @return The JWT containing all data from the provided JsonObject.
   * @throws JwtConversionException In any case of missing claims or wrong data types. 
   */
  protected IdToken convertToIdTokenClaims(final JsonObject headerObject, final JsonObject payloadObject) throws JwtConversionException
  {
    IdToken idToken;
    JsonObjectAccessor jsonAccessor;

    idToken = new IdToken(headerObject);
    jsonAccessor = new JsonObjectAccessor(payloadObject);

    try
    {
      // REQUIRED
      idToken.setIssuer(jsonAccessor.getRequiredStringValue(OpenIdConnectConstants.ID_TOKEN_ISSUER));
      idToken.setSubject(jsonAccessor.getRequiredStringValue(OpenIdConnectConstants.ID_TOKEN_SUBJECT));
      this.addAudiences(jsonAccessor, idToken, OpenIdConnectConstants.ID_TOKEN_AUDIENCE);
      idToken.setExpirationTime(this.getTimestamp(jsonAccessor, OpenIdConnectConstants.ID_TOKEN_EXPIRES, true));
      idToken.setIssuedTime(this.getTimestamp(jsonAccessor, OpenIdConnectConstants.ID_TOKEN_ISSUED_AT, true));

      // OPTIONAL
      idToken.setAuthenticationTime(this.getTimestamp(jsonAccessor, OpenIdConnectConstants.ID_TOKEN_AUTH_TIME, false));
      idToken.setNonce(jsonAccessor.getOptionalStringValue(OpenIdConnectConstants.ID_TOKEN_NONCE));
      idToken.setAuthenticationContext(jsonAccessor.getOptionalStringValue(OpenIdConnectConstants.ID_TOKEN_AUTH_CONTEXT_REF));
      this.addAuthenticationMethods(jsonAccessor, idToken, OpenIdConnectConstants.ID_TOKEN_AUTH_METHOD_REF);
      idToken.setAuthorizedParty(jsonAccessor.getOptionalStringValue(OpenIdConnectConstants.ID_TOKEN_AUTHORIZED_PARTY));

      return idToken;
    }
    catch (RuntimeException e)
    {
      throw new JwtConversionException(e, "Unable to convert given JSON object to valid JsonWebToken. %s", e.toString());
    }
  }

  protected void addAudiences(JsonObjectAccessor jsonAccessor, IdToken idToken, String fieldName) throws JwtConversionException
  {
    Object value;
    String strValue;
    JsonArray arrayValue;
    String[] strings;

    value = jsonAccessor.getRequiredRawValue(fieldName);
    if (value instanceof String)
    {
      strValue = (String)value;
      idToken.addAudiences(strValue);
    }
    if (value instanceof JsonArray)
    {
      arrayValue = (JsonArray)value;
      try
      {
        strings = SU.asStrings(arrayValue.asList(String.class));
      }
      catch (Exception ex)
      {
        throw new JwtConversionException(ex, "Provided array of \"%s\" does not only contain strings!", fieldName);
      }
      idToken.addAudiences(strings);
    }
  }

  protected void addAuthenticationMethods(JsonObjectAccessor jsonAccessor, IdToken idToken, String fieldName) throws JwtConversionException
  {
    JsonArray arrayValue;
    String[] strings;

    arrayValue = jsonAccessor.getOptionalJsonArrayValue(fieldName);
    if (arrayValue == null)
    {
      return;
    }
    try
    {
      strings = SU.asStrings(arrayValue.asList(String.class));
    }
    catch (Exception ex)
    {
      throw new JwtConversionException(ex, "Provided array of \"%s\" does not only contain strings!", fieldName);
    }
    idToken.addAuthenticationMethods(strings);
  }

}
