// ===========================================================================
// CONTENT  : CLASS OpenIdConnectTokenHandler
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.1 - 11/01/2018
// HISTORY  :
//  22/08/2014  mdu  CREATED
//  11/01/2018  mdu   changed --> removed scope validation
//
// Copyright (c) 2014-2018, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.oidc.token;

// ===========================================================================
// IMPORTS
// ===========================================================================
import static org.pfsw.oidc.OpenIdConnectConstants.*;
import static org.pfsw.oidc.internal.LoggerProvider.*;

import org.pfsw.jwt.JwtConstants;
import org.pfsw.jwt.JwtConversionException;
import org.pfsw.jwt.jws.JoseHeader;
import org.pfsw.logging.Logger2;
import org.pfsw.oidc.internal.LoggerProvider;
import org.pfsw.text.StringUtil;
import org.pfsw.text.TimeValue;
import org.pfsw.text.json.JsonObject;
import org.pfsw.text.json.SimpleJsonParser;

/**
 * Parses JSON and validates it. Then provides all token data in an instance
 * of {@link TokenData}.
 *
 * @author Manfred Duchrow
 * @version 1.1
 */
public class OpenIdConnectTokenHandler
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  private static final StringUtil SU = StringUtil.current();

  private static final String[] ALLOWED_TYPES = { "JWT", "JWS" };

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private final boolean requireSignature;
  private final boolean requireTokenType;

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  public OpenIdConnectTokenHandler()
  {
    this(true, true);
  }

  public OpenIdConnectTokenHandler(boolean requireSignature, boolean requireTokenType)
  {
    super();
    this.requireSignature = requireSignature;
    this.requireTokenType = requireTokenType;
  }

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  public TokenData parseTokenResponse(String jsonText)
  {
    TokenData tokenData;
    JsonObject jsonObject;
    IdToken idToken;
    long expirationTimestamp;

    jsonObject = this.parseJSON(jsonText);
    if (!this.isValidTokenResponse(jsonObject))
    {
      return null;
    }
    idToken = this.extractIdToken(jsonObject);
    if (idToken == null)
    {
      return null;
    }
    expirationTimestamp = System.currentTimeMillis() + (1000L * jsonObject.getInteger(TOKEN_RESP_EXPIRES_IN));
    tokenData = new TokenData(new TimeValue(expirationTimestamp));
    tokenData.setTokenType(jsonObject.getString(TOKEN_RESP_TOKEN_TYPE));
    tokenData.setAccessToken(jsonObject.getString(TOKEN_RESP_ACCESS_TOKEN));
    tokenData.setRefreshToken(jsonObject.getString(TOKEN_RESP_REFRESH_TOKEN));
    tokenData.setScope(jsonObject.getString(TOKEN_RESP_SCOPE));
    tokenData.setState(jsonObject.getString(TOKEN_RESP_STATE));
    tokenData.setIdToken(idToken);
    return tokenData;
  }

  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================
  protected JsonObject parseJSON(String jsonText)
  {
    SimpleJsonParser parser;
    JsonObject jsonObject;

    parser = new SimpleJsonParser();
    try
    {
      jsonObject = parser.parseObject(jsonText);
    }
    catch (Exception ex)
    {
      this.getLogger().infof(ex, "[%s] Failed to parse JSON!", PFOIDC0001);
      return null;
    }
    if (this.getLogger().isDebugEnabled())
    {
      this.getLogger().debugf("JsonObject, scope: %s", jsonObject.get("scope"));
    }
    return jsonObject;
  }

  protected IdToken extractIdToken(final JsonObject tokenObject)
  {
    String idTokenStr;
    IdTokenConverter converter;
    IdToken idToken;

    converter = new IdTokenConverter();
    idTokenStr = tokenObject.getString(TOKEN_RESP_ID_TOKEN);
    try
    {
      idToken = converter.convert(idTokenStr);
    }
    catch (JwtConversionException ex)
    {
      this.getLogger().infof(ex, "[%s] The JSON object is not a valid JSON Web Token (JWT). %s. %s", PFOIDC0003, ex.getMessage(), (ex.getCause() == null ? "" : ex.getCause().toString()));
      return null;
    }
    try
    {
      this.validateIdTokenHeaderType(idToken.getHeader());
      this.validateIdTokenHeaderAlgorithm(idToken.getHeader());
    }
    catch (Exception e)
    {
      this.getLogger().infof(e, "[%s] Failed to interpret id_token as JSON Web Token (JWT).", PFOIDC0002);
      return null;
    }

    return idToken;
  }

  protected boolean isValidTokenResponse(JsonObject jsonObject)
  {
    if (jsonObject == null)
    {
      return false;
    }
    try
    {
      // RFC-6749, section 5.1
      this.validateTokenType(jsonObject);
      this.validateAccessToken(jsonObject);
      this.validateExpiresIn(jsonObject);
    }
    catch (Exception e)
    {
      this.getLogger().infof("[%s] Validation of token response failed: %s", PFOIDC0004, e.getMessage());
      return false;
    }
    return true;
  }

  protected void validateExpiresIn(JsonObject jsonObject)
  {
    Integer value;

    try
    {
      value = jsonObject.getInteger(TOKEN_RESP_EXPIRES_IN);
    }
    catch (Exception ex)
    {
      throw new ValidationException(ex, "Value of %s must be a valid integer value.", TOKEN_RESP_EXPIRES_IN);
    }
    if (value <= 0)
    {
      throw new ValidationException("Token expired!");
    }
  }

  protected void validateTokenType(JsonObject jsonObject)
  {
    String value;

    value = jsonObject.getString(TOKEN_RESP_TOKEN_TYPE);
    if (!TOKEN_RESP_TOKEN_TYPE_VALUE_1.equalsIgnoreCase(value))
    {
      throw new ValidationException("Value of '%s' must be %s", TOKEN_RESP_TOKEN_TYPE, TOKEN_RESP_TOKEN_TYPE_VALUE_1);
    }
  }

  protected void validateAccessToken(JsonObject jsonObject)
  {
    String value;

    value = jsonObject.getString(TOKEN_RESP_ACCESS_TOKEN);
    if (SU.isNullOrBlank(value))
    {
      throw new ValidationException("Value of '%s' is missing.", TOKEN_RESP_ACCESS_TOKEN);
    }
  }

  protected void validateIdTokenHeaderType(JoseHeader joseHeader)
  {
    String value;

    value = joseHeader.getType();
    if (value == null)
    {
      if (this.requiresTokenType())
      {
        throw new ValidationException("Required attribute '%s' missing in token header.", JwtConstants.JOSE_TYPE);
      }
      return;
    }
    if (!SU.contains(ALLOWED_TYPES, value))
    {
      throw new ValidationException("Value of header attribute '%s' not allowed: %s", JwtConstants.JOSE_TYPE, value);
    }
  }

  protected void validateIdTokenHeaderAlgorithm(JoseHeader joseHeader)
  {
    String value;

    value = joseHeader.getAlgorithm();
    if (value == null)
    {
      throw new ValidationException("Required attribute '%s' not found in token header!", JwtConstants.JOSE_ALGORITHM);
    }
    if ("none".equals(value))
    {
      if (this.requiresSignature())
      {
        throw new ValidationException("Token must be signed! Value of header attribute '%s' not allowed: %s", JwtConstants.JOSE_ALGORITHM, value);
      }
      return;
    }
    // TODO Check here for all allowed signature algorithms
  }

  protected boolean requiresSignature()
  {
    return this.requireSignature;
  }

  protected boolean requiresTokenType()
  {
    return this.requireTokenType;
  }

  protected Logger2 getLogger()
  {
    return LoggerProvider.getComponentLogger();
  }

}
