// ===========================================================================
// CONTENT  : CLASS TokenData
// AUTHOR   : Manfred Duchrow
// VERSION  : 1.1 - 31/12/2016
// HISTORY  :
//  27/08/2014  mdu  CREATED
//  31/12/2016  mdu   added --> implements Serializable
//
// Copyright (c) 2014-2016, by MDCS. All rights reserved.
// ===========================================================================
package org.pfsw.oidc.token;

// ===========================================================================
// IMPORTS
// ===========================================================================
import java.io.Serializable;
import java.util.Date;

import org.pfsw.text.TimeValue;
import org.pfsw.util.DateUtil;

/**
 * Contains all tokens received from an OpenId Provider (OP).
 * See OpenID Connect Core 1.0, 3.1.3.3.  Successful Token Response
 * (http://openid.net/specs/openid-connect-core-1_0.html#TokenResponse).
 *
 * @author Manfred Duchrow
 * @version 1.1
 */
public class TokenData implements Serializable
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================
  private static final long serialVersionUID = -2511435218543332424L;
  private static final DateUtil DU = DateUtil.current();

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================
  private String tokenType;
  private String accessToken;
  private String refreshToken;
  private IdToken idToken;
  // Expiration time (point in time) of the Access Token
  // It is calculated from the expires_in (which is seconds after the token response was generated)
  // and the current time when the token data has been received.
  private final TimeValue expiresAt;
  private String scope;
  private String state;

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  public TokenData(TimeValue expiresAt)
  {
    super();
    this.expiresAt = expiresAt;
  } 

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  public boolean hasIdToken()
  {
    return this.getIdToken() != null;
  }

  public String getTokenType()
  {
    return this.tokenType;
  }

  public void setTokenType(String tokenType)
  {
    this.tokenType = tokenType;
  }

  public String getAccessToken()
  {
    return this.accessToken;
  }

  public void setAccessToken(String accessToken)
  {
    this.accessToken = accessToken;
  }

  public String getRefreshToken()
  {
    return this.refreshToken;
  }

  public void setRefreshToken(String refreshToken)
  {
    this.refreshToken = refreshToken;
  }

  public IdToken getIdToken()
  {
    return this.idToken;
  }

  public void setIdToken(IdToken idToken)
  {
    this.idToken = idToken;
  }

  /**
   * Returns the timestamp when this token data (particularly the access token) expires.
   */
  public TimeValue getExpiresAt()
  {
    return this.expiresAt;
  }

  public String getScope()
  {
    return this.scope;
  }
  
  public void setScope(String scope)
  {
    this.scope = scope;
  }
  
  public String getState()
  {
    return this.state;
  }
  
  public void setState(String state)
  {
    this.state = state;
  }
  
  /**
   * Returns true if the access token is expired.
   * That is, the current time is greater than the expiresAt value 
   * of this token data.
   */
  public boolean isExpired() 
  {
    return System.currentTimeMillis() > this.getExpiresAt().asMilliseconds();
  }
  
  @Override
  public String toString()
  {
    Date date = new Date(this.getExpiresAt().asMilliseconds());
    return "TokenData(accessToken: " + this.getAccessToken() + " ; expires at: " + DU.convertDateIntoZuluTime(date) + ")";
  }
}
